( cd usr/lib ; rm -rf libbluetooth.so )
( cd usr/lib ; ln -sf libbluetooth.so.2.9.4 libbluetooth.so )
( cd usr/lib ; rm -rf libbluetooth.so.2 )
( cd usr/lib ; ln -sf libbluetooth.so.2.9.4 libbluetooth.so.2 )
